# $Id: MailUser.pm 44613 2016-02-04 14:51:50Z anton $
# $URL: https://svn.uvt.nl/its-id/trunk/sources/pwdmodifier/keymaster/lib/MailUser.pm $
use strict;
use warnings FATAL => 'all';
package MailUser;
use Data::Dumper;
use UvT::Email::Template;
use UvT::PwdModifier::Conduit;
use Carp;
use Spiffy -Base;

field 'cf', -init => 'die "no cf configured\n"';
field 'params', -init => 'die "no params provided\n"';
field 'log', -init => 'die "no log provided\n"';
field 'debug' => 0;

use constant SECONDSPERDAY => 86400;

sub humanDateTime {
	# expects: epochdays
	# returns dutch AND english date
	my $timeParam = shift;
	
	my ($sec, $min, $hours, $mday, $month, $year, $wday, $yday) = localtime($timeParam);
	my $dag = qw(zondag maandag dinsdag woensdag donderdag vrijdag zaterdag)[$wday];
	my $maand = qw(januari februari maart april mei juni juli augustus september oktober november december) [$month];

	my $day = qw(Sunday Monday Tuesday Wednesday Thursday Friday Saturday)[$wday];
	my $monthName = qw(January February March April May June July August September October November December)[$month];

	$year += 1900;
	$min = sprintf("%02d", $min);

	my $dutchDate = "$mday $maand $year";
	my $dutchTime = "$hours:$min";
	my $englishDate = "$monthName $mday $year";
	my $midi = "a.m"; 
	if ($hours > 12) {
		$hours -= 12;
		$midi = "p.m";
	}

	my $englishTime = "$hours:$min $midi";
	$month++;
	my $neutralDate = "$year-$month-$mday";
	return ($neutralDate, $dutchDate, $dutchTime, $englishDate, $englishTime);
}


sub mailUser {
	my $params = $self->params;
	my $cf = $self->cf;
	my $cfh = $cf->hash();
	my $log = $self->log;

	$log->debug("MailUser::mailUser");

	unless (-x $cfh->{mailprogram}) {
		warn "No mailprogram configured, no mail sent!";
		return;
	}

	my %h = (%$cfh, %$params);

	my $uid = $self->{params}->{uid};

	my $sql = $params->{sql};
	my $anr = $params->{anr};
	my ($extramail) = $sql->getMail($anr);

	my $command = $params->{command};
	my $mailTag = lc("mail$command");
	my $xslTag = $mailTag;
	$xslTag =~ s/\d//g;
	$log->debug("mailTag: $mailTag, xslTag: $xslTag");
	my $xslFile = lc("${command}.xsl");
	$xslFile = $params->{xslFile} if $params->{xslFile};
	
	$log->debug("xslFile: \"$xslFile\"");

	if ($cf->error)	{
		my $err = $cf->error;
		chomp $err;
		die "configuration error: $err";
	}

	my $testMailSubjectPrefix = $cfh->{testMailSubjectPrefix} // '';
	$testMailSubjectPrefix .= ' ' if $testMailSubjectPrefix;
	$log->verbose("testMailSubjectPrefix: $testMailSubjectPrefix") if $testMailSubjectPrefix;

	my $subject = $testMailSubjectPrefix . $params->{subject}; 
	warn "MailUser: subject: $subject";

	my $changed_by = $params->{changed_by};
	if ($changed_by && $changed_by ne '_self_') {
		$h{requestfrom} = '';
		$log->debug( "requestfrom: $changed_by");
	}

	my $mailto = $params->{mail};
	my @maillist;

	if ($cfh->{dryrun} && $cfh->{dryrun} eq 'yes') {
		warn "Not mailing because of dryrun";
		return ;
	}

	if ($params->{stealthmode}) {
		warn "stealthmode: actuallyMailTo: anton!";
		$cfh->{actuallyMailTo} = $cfh->{silentMailAddress};
	}

	# haal multiple values voor privateEmail op
	my @privateEmail = ();
	if ($params->{privateEmail}) {
		my $entry = $params->{entry};
		@privateEmail = $entry->get_value('privateEmail');
	}

	my $tiasPrivateEmail = $params->{tiasPrivateEmail};

	$log->debug("transformer: " . "\"$cfh->{uvtEmailTransformerpath}\" \"$xslFile\"");

	my $Template = new UvT::Email::Template(
		htmltemplate => $cfh->{uvtEmailHtmltemplate},
		imagepath => $cfh->{uvtEmailImagepath},
		transformer => "$cfh->{uvtEmailTransformerpath}/$xslFile");

	my $persoon = {};

	my $url;
	$url = "$cfh->{url}?token=$params->{token}" if $params->{token}; 
	if ($url){
		$persoon->{url_en} = "$url\&language=en";
		$persoon->{url_nl} = "$url\&language=nl";
	}
	
	unless ($url){
		if ($cfh->{url}){
			$url = $cfh->{url};
			$persoon->{url_en} = "$url\?uid=$uid&language=en";
			$persoon->{url_nl} = "$url\?uid=$uid&language=nl";
		}
	}
	$persoon->{voornaam} = $params->{givenName} || $params->{initials} || $cfh->{defaultVoornaam};
	$persoon->{givenname} = $params->{givenName} || $params->{initials} || $cfh->{defaultGivenname};
	
	@$persoon{keys %$cfh} = values %$cfh;
	@$persoon{keys %$params} = values %$params;
	
	($persoon->{neutralDate}, $persoon->{datum}, $persoon->{tijd}, $persoon->{date}, $persoon->{time}) = $self->humanDateTime(time);
	$persoon->{anr} = $anr;

#	warn "TODAY", $persoon->{today} if $persoon->{today};

	# Verzamel Targetmailadressen
	my %mailHash;
	foreach my $mailTarget ($mailto, $extramail, $tiasPrivateEmail, @privateEmail) {
		next unless $mailTarget;
#		warn "mailTarget: $mailTarget";
		$mailHash{lc($mailTarget)} = undef;
	}

	my $mailTargets = join(",", keys(%mailHash));
	$persoon->{mailTarget} = $mailTargets;

	if ($self->debug) {
		warn "subject: $subject";
		warn "replyto: $cfh->{mailreplyto}";
		warn "mailfrom: $cfh->{mailfrom}";
		warn "target: $mailTargets";
	}
	
	my $adminMailAddress = '';
	if ($params->{notifyAdmin}) {
		$adminMailAddress = $params->{adminMailAddress} if $params->{adminMailAddress};
		$self->debug("NotifyAdmin: $adminMailAddress");
	}

	my $mailhandle = new FileHandle();
	
	# Wessel: sendmail kijkt alleen naar de headers als je "-t" meegeeft.
	# Schrijf een eventuele bcc maar op de envelop.
	#
	my $sent = '';
	my $envelopeTargets = $mailTargets;
	
	$envelopeTargets .= ",$cfh->{mailbcc}" if $cfh->{mailbcc};
	if ($command =~ /block/) {
		$envelopeTargets .= ",$cfh->{certmailaddress}";
		$envelopeTargets .= ",$cfh->{helpdeskmailaddress}";
	}

	my @envelopeTargetList = split(/,/, $envelopeTargets);

	my $mailErrorsTo = '';
	$mailErrorsTo = "-f $cfh->{mailerrorsto}" if $cfh->{mailerrorsto};
	$log->debug("envelopeTargets: $envelopeTargets, errorsto: $mailErrorsTo");

	my $actuallyMailTo = $params->{actuallyMailTo} // $cfh->{actuallyMailTo} // '';

	$log->debug("actuallyMailTo: $actuallyMailTo");

	foreach my $envelopeTarget (@envelopeTargetList, $adminMailAddress) {
		next unless $envelopeTarget;
		$log->debug("envelopeTarget: \"$envelopeTarget\"");
		$log->debug( "header mail targets: \"$mailTargets\"");
		if ($actuallyMailTo) {
			warn "Would mail to: $envelopeTarget, ACTUALLY mailing to: $actuallyMailTo";
			$envelopeTarget = $actuallyMailTo;
		}

		if ($envelopeTarget eq $adminMailAddress) {
			# Cert krijgt al een BCC bij block commands
			next if $command =~ /block/;
			$persoon->{adminText} = 1;
		}

		my $xmlInput = UvT::PwdModifier::Conduit::hash2xml('persoon', $persoon);

		if (open($mailhandle, '|-', $cfh->{mailprogram}, '-oi', "$mailErrorsTo", '--', $envelopeTarget)){
			binmode $mailhandle, ':utf8';		
			my $config = { 
				'Subject' => $subject,
				'Reply-to' => $cfh->{mailreplyto},
				'From' => $cfh->{mailfrom},
				'To' => $mailTargets
			};

			my $xmlOutput = $Template->format($xmlInput, %$config );

			print $mailhandle $xmlOutput;
			close $mailhandle ;
			$sent = $mailTargets;

			if ($params->{anr}) {
				warn "mail sent to $envelopeTarget ($params->{anr})";
			} else {
				warn("No anr found! mail sent anyway to $envelopeTargets!");
			}
		}  	else {
			die "Could not mail to $envelopeTarget: $!";
		}
	}
	return $sent;
}
